#include "Example.h"
#include "TRandom3.h"
#include "TMath.h"
#include "TProcessID.h"

ClassImp (Example)

Example::Example()
{
    fVerbose = 0;
    nlep = 0;
    hfile = 0;
    tree = 0;
    event = 0;
    nb = 0;
}

Example::~Example()
{}

void Example::SetExample()
{
    int branchStyle = 1;
    int split = 2;
    int compress = 1;
    if (split < 0) {branchStyle = 0; split = -1-split;}
    nb = 0;
    hfile = new TFile("Example.root","RECREATE","Example ROOT file");
    hfile->SetCompressionLevel(compress);
    tree = new TTree("Tree","JZEvent");
    tree->SetAutoSave(50000);  // autosave when 0.5 Gbyte written
    tree->SetMaxTreeSize(500000000);
    TTree::SetBranchStyle(branchStyle);
    int bufsize = 64000;
    if (split) bufsize /= 4;
    event = new JZEvent();
    TBranch *branch = tree->Branch("event", "JZEvent", &event, bufsize, split);
    branch->SetAutoDelete(kFALSE);
}

void Example::EndJob()
{
    hfile->Write();
    tree->Print();
    hfile->Close();
    delete event;
}

void Example::SetLepton(int j)
{
    TRandom *r = new TRandom3(2);
    float et = 20.*r->Rndm();
    float eta = 3.*(r->Rndm()-0.5);
    float phi = TMath::TwoPi()*(r->Rndm());
    lep[j].set_px(et*cos(phi));
    lep[j].set_py(et*sin(phi));
    lep[j].set_pz(et*tanh(eta));
    lep[j].set_pt(et);
    lep[j].set_en(et*cosh(eta));
    lep[j].set_et(et);
    lep[j].set_eta(eta);
    lep[j].set_phi(phi);
    lep[j].set_z0(120.*(r->Rndm()-0.5));
    lep[j].set_id(r->Rndm()>0.5 ? 11 : -11);
    lep[j].set_qual(0x0);
    lep[j].set_qual2(0x1);
    lep[j].set_qual3(0x3);
    lep[j].set_index(j);
    lep[j].set_mc(true);
}

void Example::FillJZEvent()
{
    int ObjectNumber = TProcessID::GetObjectCount();
    event->Clear();

    TRandom *r = new TRandom3(3);
// EventHeader
    int runnum = int(10000.*r->Rndm());
    int evtnum = int(20000.*r->Rndm());
    event->SetEventHeader(runnum,evtnum,1,0.,0.,0.,0.,1,8);
// Met
    float met = 10.+20.*r->Rndm();
    float metphi = TMath::TwoPi()*r->Rndm();
    event->SetMissingEt(met,metphi);
    float methepg = 10.+20.*r->Rndm();
    float metphihepg = metphi + TMath::Pi()*(r->Rndm()-0.5)/10.;
    event->SetMissingEtHepg(methepg,metphihepg);
// Hepg
    int nhepg = int(1000.0*r->Rndm());
    for ( int i=0; i<nhepg; i++ ) {
        JZHlep *hlep = event->AddHlep();
        int id=i; float pt=20.+20.*r->Rndm(); float eta=3.0*(r->Rndm()-0.5);
        float phi=TMath::TwoPi()*r->Rndm(); float en=pt*cosh(eta);
        int pid=i+10; float ppt=20.+20.*r->Rndm(); float peta=3.0*(r->Rndm()-0.5); float pphi=TMath::TwoPi()*r->Rndm(); float pen=ppt*cosh(eta);
//            cout << " id=" << id << " en=" << en << " pt=" << pt << " eta=" << eta << " phi=" << phi << endl;
        hlep->SetHlep(id,en,pt,eta,phi);
        hlep->SetHplep(pid,pen,ppt,peta,pphi);
    }
// Electron
    for ( int i=0; i<nlep; i++ ) {
        JZElectron *ele = event->AddElectron();
        int q = lep[i].id()/abs(lep[i].id());
        float et = lep[i].et();
        float eta = lep[i].eta();
        float phi = lep[i].phi();
        float hadem = 0.125*r->Rndm();
        float eop = 4.*r->Rndm();
        float lshr = 0.5*r->Rndm();
        float dx = 10.*(r->Rndm()-0.5);
        float dz = 20.*(r->Rndm()-0.5);
        float chi2strip = 15.*r->Rndm();
        int fidele = 1;
        int trig = 0;
        float isofrac = 0.2*r->Rndm();
        int trknum = i*10;
        int ncothits = 96;
        float trkpt = 20.*r->Rndm();
        float trkz0 = 120.*(r->Rndm()-0.5);
        float trkd0 = 0.4*(r->Rndm()-0.5);
        float cotchi2 = 120.;
        int ncotsegax5 = 4;
        int ncotsegst5 = 3;
        int ncotsegax7 = 2;
        int ncotsegst7 = 1;
        ele->SetElectron(q,i,et,eta,phi,hadem,eop,lshr,dx,dz,chi2strip,fidele,trig,isofrac,trknum,ncothits,trkpt,trkz0,trkd0,cotchi2,ncotsegax5,ncotsegst5,ncotsegax7,ncotsegst7);
    }

    if ( fVerbose>0 ) event->PrintEvent(0);
    TProcessID::SetObjectCount(ObjectNumber);

    nb += tree->Fill();
}
