#ifndef ROOT_JZEvent
#define ROOT_JZEvent

#include <iostream>
#include <iomanip>
#include <stdio.h>

#include "TObject.h"
#include "TClonesArray.h"
#include "TMath.h"
#include "TVector2.h"

using namespace std;

class TDirectory;

//------------------------------------------------------------------
class JZEventHeader : public TObject {
  private:
    int fRunNum;
    int fEvtNum;
    int fNvtx;
    float fVx;
    float fVy;
    float fVz;
    float fSumEt;
// 0-Not cosmic 1-HasCosmicRay() 2-HasOOTCaloE() 4-HasLowQFrac() 8-HasNotOutgoingPair()
    int fCosmic;
/*
    L3_name[0] = "DIELECTRON_CENTRAL_4";
    L3_name[1] = "DIMUON_CMUCMU4";
    L3_name[2] = "DIMUON_CMUPCMUP4";
    L3_name[3] = "DIMUON_CMU4_CMX4";
    L3_name[4] = "DIMUON_CMUP4_CMX4";
    L3_name[5] = "CEM4_CMU4";
    L3_name[6] = "CEM4_CMUP4";
    L3_name[7] = "CEM4_CMX4";
    L3_name[8] = "CEM4_PEM8";
    L3_name[9] = "CEM8_PEM8";
    L3_name[10] = "CMU4_PEM8";
    L3_name[11] = "CMUP4_PEM8";
    L3_name[12] = "CMX4_PEM8";
    L3_name[13] = "ELECTRON_4_LOOSE";
    L3_name[14] = "MUON_CMUP4";
*/
    int fTrigBit;

  public:
    JZEventHeader();
    virtual ~JZEventHeader() { }
//    void Clear(Option_t *option="");
    int RunNum() const { return fRunNum; }
    int EvtNum() const { return fEvtNum; }
    int Nvtx() const { return fNvtx; }
    float Vx() const { return fVx; }
    float Vy() const { return fVy; }
    float Vz() const { return fVz; }
    float SumEt() const { return fSumEt; }
    int Cosmic() const { return fCosmic; }
    void SetEvtHdr(int run, int evt, int nvtx, float vx, float vy, float vz, float sumet, int cosmic, int trigbit) { fRunNum=run; fEvtNum=evt; fNvtx=nvtx; fVx=vx; fVy=vy; fVz=vz; fSumEt=sumet; fCosmic=cosmic; fTrigBit=trigbit; }
    void Print(Option_t *option="") const;

    ClassDef(JZEventHeader,1)
};

//------------------------------------------------------------------
class JZHlep : public TObject {
  private:
    int fID;
    float fE;
    float fPt;
    float fEta;
    float fPhi;
    int fpID;
    float fpE;
    float fpPt;
    float fpEta;
    float fpPhi;
  public:
    JZHlep() { }
    virtual ~JZHlep() { Clear(); }
//    void Clear(Option_t *option="");
    int HlepID() const { return fID; }
    float HlepE() const { return fE; }
    float HlepPt() const { return fPt; }
    float HlepEta() const { return fEta; }
    float HlepPhi() const { return fPhi; }
    int HplepID() const { return fpID; }
    float HplepE() const { return fpE; }
    float HplepPt() const { return fpPt; }
    float HplepEta() const { return fpEta; }
    float HplepPhi() const { return fpPhi; }
    void SetHlep(int id, float en, float pt, float eta, float phi) { fID=id; fE=en; fPt=pt; fEta=eta; fPhi=TVector2::Phi_0_2pi(phi); }
    void SetHplep(int id, float en, float pt, float eta, float phi) { fpID=id; fpE=en; fpPt=pt; fpEta=eta; fpPhi=TVector2::Phi_0_2pi(phi); }
    void Print(Option_t *option="") const;

    ClassDef(JZHlep,1)
};

//------------------------------------------------------------------

class JZElectron : public TObject {
  private:
    int fCharge;
    int fIndex;
    float fEt;
    float fEta;
    float fPhi;
    float fHadEm;
    float fEoP;
    float fLshr;
    float fDx;
    float fDz;
    float fChi2Strip;
    int fFidEle;
    int fTrig;
    float fIsofrac;
    int fTrkNum;
    int fnCOTHits;
    float fTrkPt;
    float fTrkZ0;
    float fTrkD0;
    float fCOTChi2;
    int fnCOTSegAx5;
    int fnCOTSegSt5;
    int fnCOTSegAx7;
    int fnCOTSegSt7;
  public:
    JZElectron() {}
    virtual ~JZElectron() { Clear(); }
//    void Clear(Option_t *option="");
    int Charge() const { return fCharge; }
    int Index() const { return fIndex; }
    float Et() const { return fEt; }
    float Eta() const { return fEta; }
    float Phi() const { return fPhi; }
    float HadEm() const { return fHadEm; }
    float EoP() const { return fEoP; }
    float Lshr() const { return fLshr; }
    float Dx() const { return fDx; }
    float Dz() const { return fDz; }
    float Chi2Strip() const { return fChi2Strip; }
    int FidEle() const { return fFidEle; }
    int Trig() const { return fTrig; }
    float Isofrac() const { return fIsofrac; }
    int TrkNum() const { return fTrkNum; }
    int nCOTHits() const { return fnCOTHits; }
    float TrkPt() const { return fTrkPt; }
    float TrkZ0() const { return fTrkZ0; }
    float TrkD0() const { return fTrkD0; }
    float COTChi2() const { return fCOTChi2; }
    int nCOTSegAx5() const { return fnCOTSegAx5; }
    int nCOTSegSt5() const { return fnCOTSegSt5; }
    int nCOTSegAx7() const { return fnCOTSegAx7; }
    int nCOTSegSt7() const { return fnCOTSegSt7; }
    void SetElectron(int q, int i, float et, float eta, float phi, float hadem, float eop, float lshr, float dx, float dz, float chi2strip, int fidele, int trig, float isofrac, int trknum, int ncothits, float trkpt, float trkz0, float trkd0, float cotchi2, int ncotsegax5, int ncotsegst5, int ncotsegax7, int ncotsegst7) {
        fCharge = q;
        fIndex = i;
        fEt = et;
        fEta = eta;
        fPhi = TVector2::Phi_0_2pi(phi);
        fHadEm = hadem;
        fEoP = eop;
        fLshr = lshr;
        fDx = dx;
        fDz = dz;
        fChi2Strip = chi2strip;
        fFidEle = fidele;
        fTrig = trig;
        fIsofrac = isofrac;
        fTrkNum = trknum;
        fnCOTHits = ncothits;
        fTrkPt = trkpt;
        fTrkZ0 = trkz0;
        fTrkD0 = trkd0;
        fCOTChi2 = cotchi2;
        fnCOTSegAx5 = ncotsegax5;
        fnCOTSegSt5 = ncotsegst5;
        fnCOTSegAx7 = ncotsegax7;
        fnCOTSegSt7 = ncotsegst7;
    }
    void Print(Option_t *option="") const;

    ClassDef(JZElectron,1)
};

//------------------------------------------------------------------
class JZMuon : public TObject {
  private:
    int fCharge;
    int fIndex;
    float fPt;
    float fEta;
    float fPhi;
    float fEmEn;
    float fHadEn;
    float fChi2Match;
    float fDx;
    float fIsofrac;
    int fType;
    int fTrig;
    int fTrkNum;
    int fnCOTHits;
    float fTrkPt;
    float fTrkZ0;
    float fTrkD0;
    float fCOTChi2;
    int fnCOTSegAx5;
    int fnCOTSegSt5;
    int fnCOTSegAx7;
    int fnCOTSegSt7;
  public:
    JZMuon() {}
    virtual ~JZMuon() { Clear(); }
//    void Clear(Option_t *option="");
    int Charge() const { return fCharge; }
    int Index() const { return fIndex; }
    float Pt() const { return fPt; }
    float Eta() const { return fEta; }
    float Phi() const { return fPhi; }
    float EmEn() const { return fEmEn; }
    float HadEn() const { return fHadEn; }
    float Chi2Match() const { return fChi2Match; }
    float Dx() const { return fDx; }
    float Isofrac() const { return fIsofrac; }
    int Type() const { return fType; }  // CMU=1, CMP=2, CMUP=3, CMX=4
    int Trig() const { return fTrig; }
    int TrkNum() const { return fTrkNum; }
    int nCOTHits() const { return fnCOTHits; }
    float TrkPt() const { return fTrkPt; }
    float TrkZ0() const { return fTrkZ0; }
    float TrkD0() const { return fTrkD0; }
    float COTChi2() const { return fCOTChi2; }
    int nCOTSegAx5() const { return fnCOTSegAx5; }
    int nCOTSegSt5() const { return fnCOTSegSt5; }
    int nCOTSegAx7() const { return fnCOTSegAx7; }
    int nCOTSegSt7() const { return fnCOTSegSt7; }
    void SetMuon(int q, int i, float pt, float eta, float phi, float emen, float haden, float chi2match, float dx, float isofrac, int type, int trig, int trknum, int ncothits, float trkpt, float trkz0, float trkd0, float cotchi2, int ncotsegax5, int ncotsegst5, int ncotsegax7, int ncotsegst7) {
        fCharge = q;
        fIndex = i;
        fPt = pt;
        fEta = eta;
        fPhi = TVector2::Phi_0_2pi(phi);
        fEmEn = emen;
        fHadEn = haden;
        fChi2Match = chi2match;
        fDx = dx;
        fIsofrac = isofrac;
        fType = type;
        fTrig = trig;
        fTrkNum = trknum;
        fnCOTHits = ncothits;
        fTrkPt = trkpt;
        fTrkZ0 = trkz0;
        fTrkD0 = trkd0;
        fCOTChi2 = cotchi2;
        fnCOTSegAx5 = ncotsegax5;
        fnCOTSegSt5 = ncotsegst5;
        fnCOTSegAx7 = ncotsegax7;
        fnCOTSegSt7 = ncotsegst7;
    }
    void Print(Option_t *option="") const;

    ClassDef(JZMuon,1)
};

//------------------------------------------------------------------
class JZJet : public TObject {
  private:
    int fIndex;
    int fTrig;
    float fEt;
    float fEta;
    float fPhi;
    float fEmf;
    float fPt;
    float fJEScale3;
    float fJEScale4;
    float fJEScale5;
    float fJEScale6;
    float fJEScale7;
  public:
    JZJet() {}
    virtual ~JZJet() { Clear(); }
//    void Clear(Option_t *option="");
    int Index() const { return fIndex; }
    int Trig() const { return fTrig; }
    float Et() const { return fEt; }
    float Eta() const { return fEta; }
    float Phi() const { return fPhi; }
    float Emf() const { return fEmf; }
    float Pt() const { return fPt; }
    float JEScale3() const { return fJEScale3; }
    float JEScale4() const { return fJEScale4; }
    float JEScale5() const { return fJEScale4; }
    float JEScale6() const { return fJEScale4; }
    float JEScale7() const { return fJEScale4; }
    void SetJet(int id, int trig, float et, float eta, float phi, float emf, float pt, float scale3, float scale4, float scale5, float scale6, float scale7) {
        fIndex = id;
        fTrig = trig;
        fEt = et;
        fEta = eta;
        fPhi = TVector2::Phi_0_2pi(phi);
        fEmf = emf;
        fPt = pt;
        fJEScale3 = scale3;
        fJEScale4 = scale4;
        fJEScale5 = scale5;
        fJEScale6 = scale6;
        fJEScale7 = scale7;
    }
    void Print(Option_t *option="") const;

    ClassDef(JZJet,1)
};

//------------------------------------------------------------------
class JZIsoTrack : public TObject {
  private:
    int fTrig;
    int fTrkNum;
    int fnCOTHits;
    float fTrkPt;
    float fTrkEta;
    float fTrkPhi;
    float fTrkZ0;
    float fTrkD0;
    float fCOTChi2;
    int fnCOTSegAx5;
    int fnCOTSegSt5;
    int fnCOTSegAx7;
    int fnCOTSegSt7;
    float fSumP;
    float fSumPt;
    float fIso4;
  public:
    JZIsoTrack() {}
    virtual ~JZIsoTrack() { Clear(); }
//    void Clear(Option_t *option="");
    int Trig() const { return fTrig; }
    int TrkNum() const { return fTrkNum; }
    int nCOTHits() const { return fnCOTHits; }
    float TrkPt() const { return fTrkPt; }
    float TrkEta() const { return fTrkEta; }
    float TrkPhi() const { return fTrkPhi; }
    float TrkZ0() const { return fTrkZ0; }
    float TrkD0() const { return fTrkD0; }
    float COTChi2() const { return fCOTChi2; }
    int nCOTSegAx5() const { return fnCOTSegAx5; }
    int nCOTSegSt5() const { return fnCOTSegSt5; }
    int nCOTSegAx7() const { return fnCOTSegAx7; }
    int nCOTSegSt7() const { return fnCOTSegSt7; }
    float SumP() const { return fSumP; }
    float SumPt() const { return fSumPt; }
    float Iso4() const { return fIso4; }
    void SetIsoTrack(int trig, int trknum, int ncothits, float trkpt, float trketa, float trkphi, float trkz0, float trkd0, float cotchi2, int ncotsegax5, int ncotsegst5, int ncotsegax7, int ncotsegst7, float sump, float sumpt, float iso4) {
        fTrig = trig;
        fTrkNum = trknum;
        fnCOTHits = ncothits;
        fTrkPt = trkpt;
        fTrkEta = trketa;
        fTrkPhi = trkphi;
        fTrkZ0 = trkz0;
        fTrkD0 = trkd0;
        fCOTChi2 = cotchi2;
        fnCOTSegAx5 = ncotsegax5;
        fnCOTSegSt5 = ncotsegst5;
        fnCOTSegAx7 = ncotsegax7;
        fnCOTSegSt7 = ncotsegst7;
        fSumP = sump;
        fSumPt = sumpt;
        fIso4 = iso4;
    }
    void Print(Option_t *option="") const;

    ClassDef(JZIsoTrack,1)
};

//------------------------------------------------------------------
class JZMissingEt : public TObject {
  private:
    float fMet;
    float fPhi;
    float fMetHepg;
    float fPhiHepg;
  public:
    JZMissingEt(): fMet(0.), fPhi(0.), fMetHepg(0.), fPhiHepg(0.) { }
    virtual ~JZMissingEt() {}
//    void Clear(Option_t *option="");
    float Met() const { return fMet; }
    float Phi() const { return fPhi; }
    float MetHepg() const { return fMetHepg; }
    float PhiHepg() const { return fPhiHepg; }
    void SetMet(float met, float phi) { fMet=met; fPhi=TVector2::Phi_0_2pi(phi); }
    void SetMetHepg(float met, float phi) { fMetHepg=met; fPhiHepg=TVector2::Phi_0_2pi(phi); }
    void Print(Option_t *option="") const;

    ClassDef(JZMissingEt,1)
};

//-------------------------------------------------------------------
class JZEvent : public TObject {
  private:
    int fNHlep;
    int fNEle;
    int fNMu;
    int fNJet;
    int fNIsoTrk;
    JZEventHeader      fEvtHdr;
    JZMissingEt        fMet;
    TClonesArray       *fHleps;
    TClonesArray       *fElectrons;
    TClonesArray       *fMuons;
    TClonesArray       *fJets;
    TClonesArray       *fIsoTracks;
  public:
    JZEvent();
    virtual ~JZEvent();
    void Clear(Option_t *option="");
    void SetEventHeader(int run, int evt, int nvtx, float vx, float vy, float vz, float sumet, int cosmic, int trigbit) { fEvtHdr.SetEvtHdr(run, evt, nvtx, vx, vy, vz, sumet, cosmic, trigbit); }
    void SetMissingEt(float met, float phi) { fMet.SetMet(met, phi); }
    void SetMissingEtHepg(float met, float phi) { fMet.SetMetHepg(met, phi); }
    JZEventHeader *EvtHdr() { return &fEvtHdr; }
    JZMissingEt *MissEt() { return &fMet; }

    JZHlep *AddHlep();
    TClonesArray *HlepList() { return fHleps; }
    JZHlep *Hlep(int i) const { return (JZHlep*) fHleps->UncheckedAt(i); }

    JZElectron *AddElectron();
    TClonesArray *ElectronList() { return fElectrons; }
    JZElectron *Ele(int i) const { return (JZElectron*) fElectrons->UncheckedAt(i); }

    JZMuon *AddMuon();
    TClonesArray *MuonList() { return fMuons; }
    JZMuon *Mu(int i) const { return (JZMuon*) fMuons->UncheckedAt(i); }

    JZJet *AddJet();
    TClonesArray *JetList() { return fJets; }
    JZJet *Jt(int i) const { return (JZJet*) fJets->UncheckedAt(i); }

    JZIsoTrack *AddIsoTrack();
    TClonesArray *IsoTrackList() { return fIsoTracks; }
    JZIsoTrack *IsoTrk(int i) const { return (JZIsoTrack*) fIsoTracks->UncheckedAt(i); }

    int NHlep() const { return fNHlep; }
    int NEle() const { return fNEle; }
    int NMu() const { return fNMu; }
    int NJet() const { return fNJet; }
    int NIsoTrk() const { return fNIsoTrk; }
    void PrintEvent(int) const;

    ClassDef(JZEvent,1)
};

#endif
