#include <stdlib.h>

#include "Riostream.h"
#include "TROOT.h"
#include "TFile.h"
#include "TNetFile.h"
#include "TRandom3.h"
#include "TTree.h"
#include "TBranch.h"
#include "TClonesArray.h"
#include "TStopwatch.h"
#include "TProcessID.h"
#include "TMath.h"

#include "JZEvent.h"

//______________________________________________________________________________
int main(int argc, char **argv)
{
    int Nevents = 10;
    if (argc > 1)  Nevents = atoi(argv[1]);
    int branchStyle = 1;
    int split = 1;
    int compress = 1;
    if (split < 0) {branchStyle = 0; split = -1-split;}

    int nb = 0;
    TFile *hfile = new TFile("JZEvent.root","RECREATE","TTree benchmark ROOT file");
    hfile->SetCompressionLevel(compress);
    TTree *tree = new TTree("Tree","JZEvent");
    tree->SetAutoSave(500000000);  // autosave when 0.5 Gbyte written
    TTree::SetBranchStyle(branchStyle);
    int bufsize = 64000;
    if (split) bufsize /= 4;
    JZEvent *event = new JZEvent();
    TBranch *branch = tree->Branch("event", "JZEvent", &event, bufsize, split);
    branch->SetAutoDelete(kFALSE);

    TRandom *r = new TRandom3(1);
    int runnum = 1;
    for ( int ev=0; ev<Nevents; ev++ ) {
        int ObjectNumber = TProcessID::GetObjectCount();
        event->Clear();
//  Event
        int evtnum = int(Nevents*r->Rndm());
//        cout << runnum << " " << evtnum << endl;
        event->SetEventHeader(runnum,evtnum,1,0.,0.,0.,0.,1,8);
//        JZEventHeader *hdr = event->EvtHdr();
//        hdr->Print();
//  MissingET
        float met = 10.+20.*r->Rndm();
        float metphi = TMath::TwoPi()*r->Rndm();
        event->SetMissingEt(met,metphi);
        float methepg = 10.+20.*r->Rndm();
        float metphihepg = metphi + TMath::Pi()*(r->Rndm()-0.5)/10.;
        event->SetMissingEtHepg(methepg,metphihepg);
//        JZMissingEt *misset = event->MissEt();
//        misset->Print();
//  Hepg
        int nhepg = int(1000.0*r->Rndm());
        for ( int i=0; i<nhepg; i++ ) {
            JZHlep *hlep = event->AddHlep();
            int id=i; float pt=20.+20.*r->Rndm(); float eta=3.0*(r->Rndm()-0.5); float phi=TMath::TwoPi()*r->Rndm(); float en=pt*cosh(eta);
            int pid=i+10; float ppt=20.+20.*r->Rndm(); float peta=3.0*(r->Rndm()-0.5); float pphi=TMath::TwoPi()*r->Rndm(); float pen=ppt*cosh(eta);  //float pen=sqrt(pow(ppt*cosh(peta),2)+pow(10,2));
//            cout << " id=" << id << " en=" << en << " pt=" << pt << " eta=" << eta << " phi=" << phi << endl;
            hlep->SetHlep(id,en,pt,eta,phi);
            hlep->SetHplep(pid,pen,ppt,peta,pphi);
        }

        int nele = int(1000.0*r->Rndm());
        for ( int i=0; i<nele; i++ ) {
            JZElectron *ele = event->AddElectron();
            int q = r->Rndm()>0.5 ? 1 : -1;
            float et = 20.*r->Rndm();
            float eta = 3.*(r->Rndm()-0.5);
            float phi = TMath::TwoPi()*(r->Rndm());
            float hadem = 0.125*r->Rndm();
            float eop = 4.*r->Rndm();
            float lshr = 0.5*r->Rndm();
            float dx = 10.*(r->Rndm()-0.5);
            float dz = 20.*(r->Rndm()-0.5);
            float chi2strip = 15.*r->Rndm();
            int fidele = 1;
            int trig = 0;
            float isofrac = 0.2*r->Rndm();
            int trknum = i*10;
            int ncothits = 96;
            float trkpt = 20.*r->Rndm();
            float trkz0 = 120.*(r->Rndm()-0.5);
            float trkd0 = 0.4*(r->Rndm()-0.5);
            float cotchi2 = 120.;
            int ncotsegax5 = 4;
            int ncotsegst5 = 3;
            int ncotsegax7 = 2;
            int ncotsegst7 = 1;
            ele->SetElectron(q,i,et,eta,phi,hadem,eop,lshr,dx,dz,chi2strip,fidele,trig,isofrac,trknum,ncothits,trkpt,trkz0,trkd0,cotchi2,ncotsegax5,ncotsegst5,ncotsegax7,ncotsegst7);
        }
//        event->PrintEvent(0);
        TProcessID::SetObjectCount(ObjectNumber);
        nb += tree->Fill();
    }

    hfile->Write();
    tree->Print();
    hfile->Close();
    delete event;
    return 0;
}
